@@ -24,6 +24,7 @@
 #define GST_DEBUG_FORCE_DISABLE
 #include "gst_private.h"
 
+#include "gstatomic_impl.h"
 #include "gstdata_private.h"
 #include "gstbuffer.h"
 #include "gstmemchunk.h"
@@ -219,7 +220,8 @@ gst_buffer_new_and_alloc (guint size)
  * Returns: the new #GstBuffer, or NULL if there was an error.
  */
 GstBuffer*
-gst_buffer_new_from_pool (GstBufferPool *pool, guint64 offset, guint size)
+gst_buffer_new_from_pool (GstBufferPool *pool, 
+		          guint64 offset, guint size)
 {
   GstBuffer *buffer;
   
@@ -412,6 +414,12 @@ gst_buffer_span (GstBuffer *buf1, guint32 offset, GstBuffer *buf2, guint32 len)
   return newbuf;
 }
 
+/**
+ * gst_buffer_pool_default_free:
+ * @pool: a #GstBufferPool to free.
+ *
+ * Frees the memory associated with the bufferpool.
+ */
 void
 gst_buffer_pool_default_free (GstBufferPool *pool)
 {
